@@ -152,6 +152,7 @@ extern _dl_linux_resolve(void);
 extern int _dl_interpreter_exit(int);
 extern char * _dl_strdup(const char *);
 extern char * _dl_getenv(char * symbol, char ** envp);
+extern void _dl_unsetenv(char * symbol, char ** envp);
 extern int _dl_fixup(struct elf_resolve * tpnt);
 
 /*
@@ -210,7 +211,7 @@ void _dl_boot(int args){
   envp = (char **) aux_dat;
   while(*aux_dat) aux_dat++;  /* Skip over the envp pointers */
   aux_dat++;  /* Skip over NULL at end of envp */
-  dl_data[AT_EUID] = -1; /* check later to see if it is changed */
+  dl_data[AT_UID] = -1; /* check later to see if it is changed */
   while(*aux_dat)
     {
       unsigned int * ad1;
@@ -508,20 +509,13 @@ void _dl_boot(int args){
     }
     else
     {
-      char *cp;
-      if ((cp = _dl_getenv("LD_PRELOAD", envp)))
-	*cp = '\0';
-      if ((cp = _dl_getenv("LD_ELF_PRELOAD", envp)))
-	*cp = '\0';
-      if ((cp = _dl_getenv("LD_AOUT_PRELOAD", envp)))
-	*cp = '\0';
+      _dl_unsetenv("LD_PRELOAD", envp);
+      _dl_unsetenv("LD_ELF_PRELOAD", envp);
+      _dl_unsetenv("LD_AOUT_PRELOAD", envp);
       _dl_preload = NULL;
-      if ((cp = _dl_getenv("LD_LIBRARY_PATH", envp)))
-	*cp = '\0';
-      if ((cp = _dl_getenv("LD_ELF_LIBRARY_PATH", envp)))
-	*cp = '\0';
-      if ((cp = _dl_getenv("LD_AOUT_LIBRARY_PATH", envp)))
-	*cp = '\0';
+      _dl_unsetenv("LD_LIBRARY_PATH", envp);
+      _dl_unsetenv("LD_ELF_LIBRARY_PATH", envp);
+      _dl_unsetenv("LD_AOUT_LIBRARY_PATH", envp);
       _dl_library_path = NULL;
     }
   }
@@ -542,7 +536,6 @@ void _dl_boot(int args){
     struct elf_resolve * tpnt1;
     char * lpnt;
 
-    tcurr = _dl_loaded_modules;
     if (_dl_preload)  {
       int i;
       char *str;
@@ -582,58 +575,61 @@ void _dl_boot(int args){
 	}
       }
     }
-    do{
-      for(dpnt = (struct dynamic *) tcurr->dynamic_addr; dpnt->d_tag; dpnt++)
+
+    for (tcurr = _dl_loaded_modules; tcurr; tcurr = tcurr->next)
+    {
+      for (dpnt = (struct dynamic *)tcurr->dynamic_addr; dpnt->d_tag; dpnt++)
+      {
+	if(dpnt->d_tag == DT_NEEDED)
 	{
-	  if(dpnt->d_tag == DT_NEEDED)
+	  lpnt = tcurr->loadaddr + tcurr->dynamic_info[DT_STRTAB] + 
+	    dpnt->d_un.d_val;
+	  if (tpnt && _dl_strcmp(lpnt, tpnt->libname) == 0)
+	  {
+	    struct elf_resolve * ttmp;
+	    ttmp = _dl_loaded_modules;
+	    while (ttmp->next) 
+	      ttmp = ttmp->next;
+	    ttmp->next = tpnt;
+	    tpnt->prev = ttmp;
+	    tpnt->next = NULL;
+	    rpnt->next = 
+	      (struct dyn_elf *) _dl_malloc(sizeof(struct dyn_elf));
+	    _dl_memset (rpnt->next, 0, sizeof (*(rpnt->next)));
+	    rpnt = rpnt->next;
+	    rpnt->dyn = tpnt;
+	    tpnt->usage_count++;
+	    tpnt->symbol_scope = _dl_symbol_tables;
+	    tpnt = NULL;
+	    continue;
+	  }
+	  if (!(tpnt1 = _dl_load_shared_library(tcurr, lpnt)))
+	  {
+	    if (_dl_trace_loaded_objects)
+	      _dl_fdprintf(1, "\t%s => not found\n", lpnt);
+	    else
 	    {
-	      lpnt = tcurr->loadaddr + tcurr->dynamic_info[DT_STRTAB] + 
-		dpnt->d_un.d_val;
-	      if(tpnt && _dl_strcmp(lpnt, tpnt->libname) == 0) {
-		struct elf_resolve * ttmp;
-		ttmp = _dl_loaded_modules;
-		while(ttmp->next) ttmp = ttmp->next;
-		ttmp->next = tpnt;
-		tpnt->prev = ttmp;
-		tpnt->next = NULL;
-		rpnt->next = 
-		   (struct dyn_elf *) _dl_malloc(sizeof(struct dyn_elf));
-		_dl_memset (rpnt->next, 0, sizeof (*(rpnt->next)));
-		rpnt = rpnt->next;
-		rpnt->dyn = tpnt;
-		tpnt->usage_count++;
-		tpnt->symbol_scope = _dl_symbol_tables;
-		tpnt = NULL;
-		continue;
-	      }
-	      if(!(tpnt1 = _dl_load_shared_library(tcurr, lpnt))) {
-		if (_dl_trace_loaded_objects)
-		  _dl_fdprintf(1, "\t%s => not found\n", lpnt);
-		else
-		{
-		  _dl_fdprintf(2, "%s: can't load library '%s'\n",
-			       _dl_progname, lpnt);
-		  _dl_exit(16);
-		}
-	      }
-	      else
-	      {
-		if (_dl_trace_loaded_objects && !tpnt1->usage_count)
-		  _dl_fdprintf(1, "\t%s => %s\n", lpnt, tpnt1->libname);
-		rpnt->next = 
-		  (struct dyn_elf *) _dl_malloc(sizeof(struct dyn_elf));
-		_dl_memset (rpnt->next, 0, sizeof (*(rpnt->next)));
-		rpnt = rpnt->next;
-		tpnt1->usage_count++;
-		tpnt1->symbol_scope = _dl_symbol_tables;
-		tpnt1->libtype = elf_lib;
-		rpnt->dyn = tpnt1;
-	      }
+	      _dl_fdprintf(2, "%s: can't load library '%s'\n",
+			   _dl_progname, lpnt);
+	      _dl_exit(16);
 	    }
+	  }
+	  else
+	  {
+	    if (_dl_trace_loaded_objects && !tpnt1->usage_count)
+	      _dl_fdprintf(1, "\t%s => %s\n", lpnt, tpnt1->libname);
+	    rpnt->next = 
+	      (struct dyn_elf *) _dl_malloc(sizeof(struct dyn_elf));
+	    _dl_memset (rpnt->next, 0, sizeof (*(rpnt->next)));
+	    rpnt = rpnt->next;
+	    tpnt1->usage_count++;
+	    tpnt1->symbol_scope = _dl_symbol_tables;
+	    tpnt1->libtype = elf_lib;
+	    rpnt->dyn = tpnt1;
+	  }
 	}
-      
-      tcurr = tcurr->next;
-    } while(tcurr);
+      }
+    }
   }
 
 #ifdef USE_CACHE
@@ -689,7 +685,7 @@ void _dl_boot(int args){
    * directives work correctly */
 
 
-  goof = _dl_fixup(_dl_loaded_modules);
+  goof = _dl_loaded_modules ? _dl_fixup(_dl_loaded_modules) : 0;
 
 
   /* Some flavors of SVr4 do not generate the R_*_COPY directive,
@@ -697,7 +693,8 @@ void _dl_boot(int args){
    Solaris gets this one right, from what I understand.  */
 
 
-  goof +=  _dl_copy_fixups(_dl_symbol_tables);
+  if (_dl_symbol_tables)
+    goof += _dl_copy_fixups(_dl_symbol_tables);
 
   if(goof || _dl_trace_loaded_objects) _dl_exit(0);
 
@@ -865,33 +862,44 @@ void * _dl_malloc(int size) {
   return retval;
 }
 
-char * _dl_getenv(char * symbol, char ** envp)
+char * _dl_getenv(char *symbol, char **envp)
 {
-  char * pnt;
-  char * pnt1;
+  char *pnt;
+  char *pnt1;
   while ((pnt = *envp++)) {
     pnt1 = symbol;
-    while(*pnt && *pnt1 && *pnt == *pnt1) {pnt1++; pnt++;}
-    if(!*pnt || *pnt != '=' || *pnt1) continue;
+    while (*pnt && *pnt == *pnt1)
+      pnt1++, pnt++;
+    if (!*pnt || *pnt != '=' || *pnt1)
+      continue;
     return pnt+1;
   }
   return 0;
 }
 
-char * _dl_strdup(const char * string){
-  void * retval;
-  char * pnt;
+void _dl_unsetenv(char *symbol, char **envp)
+{
+  char *pnt;
+  char *pnt1;
+  char **newenvp = envp;
+  for (pnt = *envp; pnt; pnt = *++envp) {
+    pnt1 = symbol;
+    while (*pnt && *pnt == *pnt1)
+      pnt1++, pnt++;
+    if(!*pnt || *pnt != '=' || *pnt1)
+      *newenvp++ = *envp;
+  }
+  *newenvp++ = *envp;
+  return;
+}
 
-  pnt = retval = _dl_malloc_addr;
-  while(*string)
-    *pnt++ = *string++;
-  *pnt++ = 0;
-  _dl_malloc_addr = pnt;
-  /*
-   * Align memory to 4 byte boundary.  Some platforms require this, others
-   * simply get better performance.
-   */
-  _dl_malloc_addr = (char *) (((unsigned int) _dl_malloc_addr + 3) & ~(3));
+char * _dl_strdup(const char * string){
+  char * retval;
+  int len;
+
+  len = _dl_strlen(string);
+  retval = _dl_malloc(len + 1);
+  _dl_strcpy(retval, string);
   return retval;
 }
 
