@@ -5,7 +5,7 @@
  */
 
 #include <stdio.h>
-#include <dlfcn.h>
+#include "dlfcn.h"
 #include <link.h>
 #include <stdlib.h>
 #include <sys/mman.h>
@@ -19,7 +19,6 @@ extern struct r_debug * _dl_debug_addr;
 
 extern void * (*_dl_malloc_function)(size_t size);
 
-static struct dyn_elf * handles;
 static int do_fixup(struct elf_resolve * tpnt, int flag);
 static int do_dlclose(void *, int need_fini);
 
@@ -58,7 +57,7 @@ static void dl_cleanup()
 {
 	struct dyn_elf*		d;
 	
-	for (d = handles; d; d = d->next_handle)
+	for (d = _dl_handles; d; d = d->next_handle)
 		if (d->dyn->libtype == loaded_file && d->dyn->dynamic_info[DT_FINI])
 		{
 			(*((int(*)(void))(d->dyn->loadaddr + d->dyn->dynamic_info[DT_FINI])))();
@@ -95,7 +94,11 @@ void * _dlopen(char * libname, int flag)
 
 	/*
 	 * Try and locate the module we were called from - we
-	 * need this so that we get the correct RPATH.
+	 * need this so that we get the correct RPATH.  Note that
+	 * this is the current behavior under Solaris, but the
+	 * ABI+ specifies that we should only use the RPATH from
+	 * the application.  Thus this may go away at some time
+	 * in the future.
 	 */
 	tfrom = NULL;
 	for(dpnt = _dl_symbol_tables; dpnt; dpnt = dpnt->next)
@@ -120,10 +123,11 @@ void * _dlopen(char * libname, int flag)
 	   (struct dyn_elf *) malloc(sizeof(struct dyn_elf));
 	_dl_memset (rpnt, 0, sizeof(*rpnt));
 	rpnt->dyn = tpnt;
+	rpnt->flags = flag;
 	if (!tpnt->symbol_scope) tpnt->symbol_scope = dyn_chain;
 	
-	rpnt->next_handle = handles;
-	handles = rpnt;
+	rpnt->next_handle = _dl_handles;
+	_dl_handles = rpnt;
 
 	/*
 	 * OK, we have the requested file in memory.  Now check for
@@ -267,8 +271,8 @@ void * _dlsym(void * vhandle, char * name)
 
 	handle = (struct dyn_elf *) vhandle;
 
-	if (vhandle != RTLD_NEXT) {
-		for(rpnt = handle; rpnt; rpnt = rpnt->next_handle)
+	if (vhandle != RTLD_NEXT && handle != _dl_symbol_tables) {
+		for(rpnt = _dl_handles; rpnt; rpnt = rpnt->next_handle)
 			if(rpnt == handle) break;
 		if (!rpnt) {
 			_dl_error_number = DL_BAD_HANDLE;
@@ -277,6 +281,10 @@ void * _dlsym(void * vhandle, char * name)
 	}
 		
 	ret = _dl_find_hash(name, handle, 1, NULL, 1);
+
+       /*
+        * Nothing found.
+        */
 	if (!ret)
 	{
 		_dl_error_number = DL_NO_SYMBOL;
@@ -303,7 +311,7 @@ static int do_dlclose(void * vhandle, int need_fini)
 
 	handle = (struct dyn_elf *) vhandle;
 	rpnt1 = NULL;
-	for(rpnt = handles; rpnt; rpnt = rpnt->next_handle)
+	for(rpnt = _dl_handles; rpnt; rpnt = rpnt->next_handle)
 	{
 		if(rpnt == handle) {
 			break;
@@ -342,7 +350,7 @@ static int do_dlclose(void * vhandle, int need_fini)
 	if(rpnt1)
 	    rpnt1->next_handle = rpnt->next_handle;
 	else
-	    handles = rpnt->next_handle;
+	    _dl_handles = rpnt->next_handle;
 	
 	/* OK, this is a valid handle - now close out the file */
 	for(rpnt = handle; rpnt; rpnt = rpnt1)
@@ -453,6 +461,7 @@ static int foobar1 = (int)foobar; /* Use as pointer */
 #if 1
 #pragma weak _dl_find_hash = foobar
 #pragma weak _dl_symbol_tables = foobar1
+#pragma weak _dl_handles = foobar1
 #pragma weak _dl_loaded_modules = foobar1
 #pragma weak _dl_debug_addr = foobar1
 #pragma weak _dl_error_number = foobar1
@@ -467,6 +476,7 @@ static int foobar1 = (int)foobar; /* Use as pointer */
 #else
 __asm__(".weak _dl_find_hash; _dl_find_hash = foobar");
 __asm__(".weak _dl_symbol_tables; _dl_symbol_tables = foobar1");
+__asm__(".weak _dl_handles; _dl_handles = foobar1");
 __asm__(".weak _dl_loaded_modules; _dl_loaded_modules = foobar1");
 __asm__(".weak _dl_debug_addr; _dl_debug_addr = foobar1");
 __asm__(".weak _dl_error_number; _dl_error_number = foobar1");
@@ -507,7 +517,7 @@ void _dlinfo()
 		fprintf(stderr, "\t%8.8x %s\n", (unsigned)rpnt->dyn,
 			rpnt->dyn->libname);
 
-	for (hpnt = handles; hpnt; hpnt = hpnt->next_handle)
+	for (hpnt = _dl_handles; hpnt; hpnt = hpnt->next_handle)
 	{
 		fprintf(stderr, "Modules for handle %x\n", (unsigned)hpnt);
 		for(rpnt = hpnt; rpnt; rpnt = rpnt->next)
