@@ -83,7 +83,8 @@ int _dl_map_cache(void)
 
   if (_dl_memcmp (header->version, LDSO_CACHE_VER, LDSO_CACHE_VER_LEN))
   {
-    _dl_fdprintf(2, "%s: cache '%s' has wrong version\n", _dl_progname, LDSO_CACHE);
+    _dl_fdprintf(2, "%s: cache '%s' is not version %s\n",
+		 _dl_progname, LDSO_CACHE, LDSO_CACHE_VER);
     goto fail;
   }
 
@@ -115,10 +116,10 @@ int _dl_unmap_cache(void)
 unsigned int _dl_error_number;
 unsigned int _dl_internal_error_number;
 
-struct elf_resolve * _dl_load_shared_library(struct dyn_elf * dpnt,
+struct elf_resolve * _dl_load_shared_library(struct elf_resolve * tpnt,
 	char * full_libname){
   char * pnt, *pnt1, *pnt2;
-  struct elf_resolve * tpnt, *tpnt1;
+  struct elf_resolve *tpnt1;
   char mylibname[1024];
   char * libname;
 
@@ -154,6 +155,31 @@ struct elf_resolve * _dl_load_shared_library(struct dyn_elf * dpnt,
     goto goof;
   }
 
+  /*
+   * The ABI specifies that RPATH is searched before LD_*_PATH or
+   * the default path of /usr/lib.
+   * Check in rpath directories 
+   */
+  if( tpnt != NULL )
+    {
+      pnt1 = (char *) tpnt->dynamic_info[DT_RPATH];
+      if(pnt1) {
+	pnt1 += (unsigned int) tpnt->loadaddr + tpnt->dynamic_info[DT_STRTAB];
+	while(*pnt1){
+	  pnt2 = mylibname;
+	  while(*pnt1 && *pnt1 != ':') *pnt2++ = *pnt1++;
+	  if(pnt2[-1] != '/') *pnt2++ = '/';
+	  pnt = libname;
+	  while(*pnt) *pnt2++  = *pnt++;
+	  *pnt2++ = 0;
+	  tpnt1 = _dl_load_elf_shared_library(mylibname, 0);
+	  if(tpnt1) return tpnt1;
+	  if(*pnt1 == ':') pnt1++;
+	}
+      }
+    }
+  
+
   /* Check in LD_{ELF_}LIBRARY_PATH, if specified and allowed */
   pnt1 = _dl_library_path;
   if (pnt1 && *pnt1) {
@@ -170,6 +196,12 @@ struct elf_resolve * _dl_load_shared_library(struct dyn_elf * dpnt,
     }
   }
 
+
+  /*
+   * Where should the cache be searched?  There is no such concept in the
+   * ABI, so we have some flexibility here.  For now, search it before
+   * the default path of /usr/lib.
+   */
 #ifdef USE_CACHE
   if (_dl_cache_addr != NULL && _dl_cache_addr != (caddr_t)-1)
   {
@@ -188,26 +220,6 @@ struct elf_resolve * _dl_load_shared_library(struct dyn_elf * dpnt,
   }
 #endif
 
-  /* Check in rpath directories */
-  for(; dpnt; dpnt = dpnt->next) {
-    tpnt = dpnt->dyn;
-    pnt1 = (char *) tpnt->dynamic_info[DT_RPATH];
-    if(pnt1) {
-      pnt1 += (unsigned int) tpnt->loadaddr + tpnt->dynamic_info[DT_STRTAB];
-      while(*pnt1){
-	pnt2 = mylibname;
-	while(*pnt1 && *pnt1 != ':') *pnt2++ = *pnt1++;
-	if(pnt2[-1] != '/') *pnt2++ = '/';
-	pnt = libname;
-	while(*pnt) *pnt2++  = *pnt++;
-	*pnt2++ = 0;
-	tpnt1 = _dl_load_elf_shared_library(mylibname, 0);
-	if(tpnt1) return tpnt1;
-	if(*pnt1 == ':') pnt1++;
-      }
-    }
-  }
-
   /* Check in /usr/lib */
 #ifdef IBCS_COMPATIBLE
   pnt1 = "/usr/i486-sysv4/lib/";
@@ -261,12 +273,11 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
   char header[4096];
   int dynamic_info[24];
   int * lpnt;
-  char  * zfile = "/dev/zero";
   unsigned int libaddr;
   unsigned int minvma=0xffffffff, maxvma=0;
   
   int i;
-  int infile, zerofile;
+  int infile;
 
   /* If this file is already loaded, skip this step */
   tpnt = _dl_check_hashed_files(libname);
@@ -286,13 +297,6 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
     _dl_internal_error_number = DL_ERROR_NOFILE;
     return NULL;
   }
-  zerofile = _dl_open(zfile, O_RDONLY);
-  if(zerofile < 0) {
-    _dl_fdprintf(2, "%s: can't open '/dev/zero'\n", _dl_progname);
-    _dl_internal_error_number = DL_ERROR_NOZERO;
-    _dl_close(infile);
-    return NULL;
-  }; 
  
   _dl_read(infile, header, sizeof(header));
   epnt = (struct elfhdr *) header;
@@ -303,7 +307,6 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
     _dl_fdprintf(2, "%s: '%s' is not an ELF file\n", _dl_progname, libname);
     _dl_internal_error_number = DL_ERROR_NOTELF;
     _dl_close(infile);
-    _dl_close(zerofile);
     return NULL;
   };
   
@@ -317,7 +320,6 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
     _dl_fdprintf(2, "%s: '%s' is not an ELF executable for " ELF_TARGET "\n",
 		 _dl_progname, libname);
     _dl_close(infile);
-    _dl_close(zerofile);
     return NULL;
   };
 
@@ -353,19 +355,18 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
   maxvma=(maxvma+0xfffU)&~0xfffU;
   minvma=minvma&~0xffffU;
   
-  flags = MAP_PRIVATE;
-  if(!piclib) flags|=MAP_FIXED;
+  flags = MAP_PRIVATE /*| MAP_DENYWRITE*/;
+  if(!piclib) flags |= MAP_FIXED;
   
   status = (char *) _dl_mmap((char *) (piclib?0:minvma),
 			  maxvma-minvma, 
 			  PROT_NONE, 
-			  flags, zerofile,
+			  flags | MAP_ANONYMOUS, -1,
 			  0);
   if(_dl_mmap_check_error(status)) {
     _dl_fdprintf(2, "%s: can't map '/dev/zero'\n", _dl_progname);
     _dl_internal_error_number = DL_ERROR_MMAP_FAILED;
     _dl_close(infile);
-    _dl_close(zerofile);
      return NULL;
   };
   libaddr=(unsigned int)status;
@@ -401,7 +402,6 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
 	    _dl_internal_error_number = DL_ERROR_MMAP_FAILED;
 	    _dl_munmap((char *)libaddr, maxvma-minvma);
 	    _dl_close(infile);
-	    _dl_close(zerofile);
 	    return NULL;
 	};
 	
@@ -420,7 +420,7 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
 	  status = (char *) _dl_mmap((char *) map_size + libaddr, 
 			    ppnt->p_vaddr + ppnt->p_memsz - map_size,
 			    LXFLAGS(ppnt->p_flags),
-			    flags, zerofile, 0);
+			    flags | MAP_ANONYMOUS, -1, 0);
       } else
 	status = (char *) _dl_mmap((char *) (ppnt->p_vaddr & 0xfffff000) + 
 				   libaddr, 
@@ -433,7 +433,6 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
 	_dl_internal_error_number = DL_ERROR_MMAP_FAILED;
 	_dl_munmap((char *)libaddr, maxvma-minvma);
 	_dl_close(infile);
-	_dl_close(zerofile);
 	return NULL;
       };
       /* if(libaddr == 0 && piclib) {
@@ -443,7 +442,6 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
     };
     ppnt++;
   };
-  _dl_close(zerofile);
   _dl_close(infile);
   
   /* For a non-PIC library, the addresses are all absolute */
