@@ -146,11 +146,17 @@ char *is_shlib(char *dir, char *name, int *type)
 	    warn("can't lstat %s (%s), skipping", buff, strerror(errno));
 	else if (S_ISLNK(statbuf.st_mode) && stat(buff, &statbuf))
 	{
-	    if (verbose)
-	        warn("%s is a stale symlink, removing", buff);
-	    if (unlink(buff))
-	        warn("can't unlink %s (%s), skipping", buff,
-		     strerror(errno));
+	    /* we've got a stale link.  if there is no version number, 
+	       assume it's for ld and leave it alone, otherwise try
+	       to remove it */
+	    if ((cp2 = strstr(name, ".so")) && cp2[3])
+	    {
+	        if (verbose)
+		    warn("%s is a stale symlink, removing", buff);
+		if (unlink(buff))
+		   warn("can't unlink %s (%s), skipping", buff,
+			strerror(errno));
+	    }
 	}
 	else if (!S_ISREG(statbuf.st_mode))
 	    warn("%s is not a regular file, skipping", buff, strerror(errno));
